package org.drupal.project.recommender;

import org.drupal.project.computing.*;
import org.drupal.project.computing.common.PingMe;
import org.drupal.project.recommender.algorithm.Item2Item;
import org.drupal.project.recommender.algorithm.Item2ItemIncrement;
import org.drupal.project.recommender.algorithm.SVD;
import org.drupal.project.recommender.algorithm.User2User;


@Identifier("recommender")
public class DefaultApplication extends DApplication {

    @Override
    protected void buildCommandRegistry() {
        registerCommand(PingMe.class);
        registerCommand(User2User.class);
        registerCommand(Item2Item.class);
        registerCommand(Item2ItemIncrement.class);
        registerCommand(SVD.class);
    }

    @Override
    protected void initDrupalSite() {
        try {
            DConfig config = new DConfig();
            DDatabase database = new DDatabase(config.getDbProperties());
            drupalSite = new DSqlSite(database);
        } catch (DConnectionException e) {
            throw new DRuntimeException(e);
        } catch (DConfigException e) {
            throw new DRuntimeException(e);
        }
    }

    /**
     * Get the multi-thread settings for recommender applications.
     * This is not dependent on each individual command, therefore it's implemented here.
     *
     * @return The threads number to use when computing recommendations. 0 means no multi-threading is supported
     */
    public int getThreadNumber() {
        int num = 1;
        try {
            num = Integer.parseInt(getConfig().getProperty("drupal.recommender.threads", "1"));
        } catch (Throwable e) {
            logger.info("Could not read drupal.recommender.threads. Use single thread instead.");
        }
        assert num >= 1;
        return num;
    }

    public static void main(String[] args) {
        DApplication app = new DefaultApplication();
        app.launchFromShell(args);
    }

}
