package org.drupal.project.recommender;

import org.apache.commons.lang.StringUtils;
import org.drupal.project.computing.*;

import java.util.Map;

/**
 * This is the generic class for all Recommender algorithms.
 * Sub-classes need to implement execute() and parseConfig()
 */
abstract public class RecommenderAlgorithm extends DCommand {

    protected String recommenderName;
    protected DDatabase database;
    protected long updatedTimestamp;  // current timestamp when this algorithm runs.

    // to save back to the computing_records
    protected int numUsers;
    protected int numItems;
    protected int numSimilarity;
    protected int numPrediction;


    @Override
    public void enterRecord(DRecord record) {
        database = ((DSqlSite) getDrupalSite()).getDatabase();
        assert database != null;

        recommenderName = record.getIds1();
        if (StringUtils.isBlank(recommenderName)) {
            throw new DRuntimeException(new DConfigException("Recommender name cannot be empty."));
        }

        String jsonParams = record.getInputjson();
        if (StringUtils.isEmpty(jsonParams)) {
            throw new DRuntimeException(new DConfigException("Cannot initialize Recommender without parameters."));
        }

        Map<String, Object> params;
        try {
            params = (Map<String, Object>) (DUtils.getInstance().fromJson(jsonParams));
        } catch (ClassCastException e) {
            throw new DRuntimeException(new DConfigException("Recommender configuration errors."));
        }

        try {
            parseConfig(params);
        } catch (DConfigException e) {
            throw new DRuntimeException(e);
        }
    }

    abstract public void parseConfig(Map<String, Object> params) throws DConfigException;

    @Override
    protected void beforeExecute() {
        super.beforeExecute();
        updatedTimestamp = record.getStart();
    }

    /**
     * Save some results back to the record.
     * @param record
     */
    @Override
    public void keepRecord(DRecord record) {
        record.setNumber1((float) numUsers);
        record.setNumber2((float) numItems);
        record.setNumber3((float) numSimilarity);
        record.setNumber4((float) numPrediction);
    }

}
