package org.drupal.project.recommender.algorithm;

import org.apache.mahout.cf.taste.common.TasteException;
import org.apache.mahout.cf.taste.impl.neighborhood.NearestNUserNeighborhood;
import org.apache.mahout.cf.taste.impl.recommender.GenericBooleanPrefUserBasedRecommender;
import org.apache.mahout.cf.taste.impl.recommender.GenericUserBasedRecommender;
import org.apache.mahout.cf.taste.recommender.UserBasedRecommender;
import org.drupal.project.computing.DConnectionException;
import org.drupal.project.recommender.MahoutRecommender;
import org.drupal.project.recommender.RecommendedEntity;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 * User: danithaca
 * Date: 9/25/12
 * Time: 4:19 PM
 */
public class User2User extends MahoutRecommender {
    // TODO: make this configurable.
    private final int NEAREST_N = 20;   // this is only used in recommendation. the number of saved similar users is still controlled by appMaxKeep
    private final double MIN_SIMILARITY = 0.1;

    @Override
    protected void initRecommender() throws TasteException {
        NearestNUserNeighborhood neighbor = new NearestNUserNeighborhood(NEAREST_N, MIN_SIMILARITY, userSimilarity, dataModel);
        if (preferenceType == PreferenceType.BOOLEAN) {
            recommender = new GenericBooleanPrefUserBasedRecommender(dataModel, neighbor, userSimilarity);
        } else if (preferenceType == PreferenceType.SCORE) {
            recommender = new GenericUserBasedRecommender(dataModel,neighbor, userSimilarity);
        } else {
            assert false;
        }
    }

    @Override
    protected void computeSimilarity() throws TasteException, SQLException, DConnectionException {
        numSimilarity = genericComputeSave(defaultUserSimilarityHandler, dataModel.getUserIDs(), dataModel.getNumUsers(), "similarity", true);
    }

    @Override
    protected void computePrediction() throws TasteException, DConnectionException, SQLException {
        numPrediction = genericComputeSave(defaultPredictionHandler, dataModel.getUserIDs(), dataModel.getNumUsers(), "prediction", true);
    }


    protected EntityHandler defaultUserSimilarityHandler = new EntityHandler() {
        @Override
        public List<RecommendedEntity> getRecommendedEntity(long entityID) throws TasteException {
            List<RecommendedEntity> recommended = new ArrayList<RecommendedEntity>();
            long[] similarUserIDs = ((UserBasedRecommender) recommender).mostSimilarUserIDs(entityID, maxKeep);
            for (long userID : similarUserIDs) {
                recommended.add(new RecommendedEntity(userID, new Float(userSimilarity.userSimilarity(entityID, userID))));
            }
            return recommended;
        }
        @Override
        public boolean validateScore(double score) {
            return validateSimilarityScore(score);
        }
    };
}
